library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity servo is
    Port ( CLOCK_6MHZ : in  STD_LOGIC;
           LINKS : in  STD_LOGIC;
           RECHTS : in  STD_LOGIC;
           PWM : out  STD_LOGIC);
end servo;

architecture Verhalten_Servo of servo is
    --Zähler, der die Clock-Signale von 6MHz zählt.
	 --Ein PWM-Puls wird mit 100Hz erzeugt.
	 --Das sind 60000 Zählschritte.
	 --Es verbleibt also bei 100Hz PWM-Signal eine Bandbreite von 60000 Schritten 
	 --für die PWM-Breite.
	 --Da der nutzbare Bereich zwischen 1 und 2 ms liegt, 
	 --darf zwischen 6000 und 12000 Schritten variiert werden, um die Servoposition einzustellen.
    signal zaehler    : integer range 0 to 59999 := 0;
    signal logikpegel : std_logic := '0';
	 signal pwm_breite : integer range 0 to 12000 := 9000; --Mittelstellung als Initialwert.
begin
    --Auch Prozessse sind konkurrierende Elemente.
	 --Verschiedene prozesshafte Zusammenhänge können
	 --unabhängig voneinander formuliert werden:

    process(CLOCK_6MHZ) begin
	     if (CLOCK_6MHZ'event and CLOCK_6MHZ='1')	then	  
            if (zaehler<59999) then
                zaehler <= zaehler+1;
            else
                zaehler <= 0;
            end if;		  
		  end if;
    end process;

    process(zaehler) begin
	     if (zaehler < pwm_breite) then
            logikpegel <= '0'; --Optokoppler invertiert das Signal!
		  else 
            logikpegel <= '1';
		  end if;
    end process;
	 
    process(LINKS,RECHTS) begin
		  if (LINKS='0') then --invertierte Logik!
		      pwm_breite <= 6000;
		  elsif (RECHTS='0') then
		      pwm_breite <= 12000;
		  else	
		      pwm_breite <= 9000;
		  end if;		
    end process;
	 
	 PWM <= logikpegel;
end Verhalten_Servo;

